@@ -14,341 +14,824 @@ package Curses::UI;
 # If we do not know a terminal type, then imply VT100.
 BEGIN { $ENV{TERM} = 'vt100' unless defined $ENV{TERM} }
 
-use Carp qw(confess);
 use Curses;
 use Curses::UI::Container;
 use Curses::UI::Common;
+use Curses::UI::Language;
+use FileHandle;
 use Term::ReadKey;
+require Exporter;
 
-use vars qw($VERSION @ISA);
-$VERSION = "0.63";
-@ISA = qw(Curses::UI::Container Curses::UI::Common);
+use vars qw(
+    $VERSION 
+    @ISA
+    @EXPORT
+);
 
-$Curses::UI::resizing = 0; 
-$Curses::UI::resizetime = undef; 
+$VERSION = "0.71";
+
+@EXPORT = qw(
+    MainLoop
+);
+
+@ISA = qw(
+    Curses::UI::Container 
+    Curses::UI::Common
+);
+
+$Curses::UI::rootobject       = undef;
+$Curses::UI::debug            = 0; 
 $Curses::UI::screen_too_small = 0; 
+$Curses::UI::initialized      = 0;
 
-$SIG{'WINCH'} = \&resize_window;
-sub resize_window()
-{
-	$Curses::UI::resizing++;
-	$Curses::UI::resizetime = time();
-	$SIG{'WINCH'} = \&resize_window;
-}
+# Detect ncurses functionality.
+$Curses::UI::ncurses_mouse    = (Curses->can("NCURSES_MOUSE_VERSION") and
+                                 NCURSES_MOUSE_VERSION() >= 1);
 
-sub check_for_resize()
-{
-	my $this = shift;
-
-        # The last resize signal should be received more than a
-        # second ago. This mechanism is used to catch window
-        # managers that send a whole bunch of signals to the
-        # application if the screen resizes. Only the last one
-        # will count.
-
-        if ($Curses::UI::resizing and
-            ($Curses::UI::resizetime <= (time()-1)))
-        {
-                $Curses::UI::resizing = 0;
-                $Curses::UI::screen_too_small = 0;
-                $this->root->layout_from_scratch;
-                $this->root->rebuild_from_scratch;
-        }
-	
-	return $this;
-}
+# ----------------------------------------------------------------------
+# Constructor
+# ----------------------------------------------------------------------
 
 sub new()
 {
-	my $class = shift;
-	my %args = (
-		-compat        => 0,
-		-clear_on_exit => 0,
-		@_,
-	);
-	my $this = bless { %args }, $class;
-	$this->layout();	
-	return $this;
+    my $class = shift;
+
+    my %userargs = @_;
+    keys_to_lowercase(\%userargs);
+
+    my %args = (
+        -compat        => 0,     # Use compatibility mode?
+        -clear_on_exit => 0,     # Clear screen if program exits?
+        -cursor_mode   => 0,     # What is the current cursor_mode?
+	-debug         => undef, # Turn on debugging mode?
+	-language      => undef, # Which language to use?
+
+        %userargs,
+
+	-read_timeout  => -1,    # full blocking read by default
+    );
+
+    $Curses::UI::debug = $args{-debug} 
+        if defined $args{-debug};
+
+    $Curses::UI::rootobject->fatalerror(
+        "You can only initiate one Curses::UI rootobject!\n"
+    ) if defined $Curses::UI::rootobject;
+
+    my $this = bless { %args }, $class;
+    $Curses::UI::rootobject = $this;
+
+    my $lang = new Curses::UI::Language($this->{-language});
+    $this->lang($lang);
+    print STDERR "DEBUG: Loaded language: $lang->{-lang}\n"
+	if $Curses::UI::debug;
+
+    $this->layout();    
+
+    return $this;
 }
 
-sub compat(;$)        { shift()->accessor('-compat', shift()) }
-sub clear_on_exit(;$) { shift()->accessor('-clear_on_exit', shift()) }
+# ----------------------------------------------------------------------
+# Destructor
+# ----------------------------------------------------------------------
+
+DESTROY 
+{ 
+    my $this = shift;
+    endwin();
+    $Curses::UI::rootobject = undef;
+
+    if ($this->{-clear_on_exit})
+    {
+        my $save_path = $ENV{PATH};
+        $ENV{PATH} = "/bin:/usr/bin";
+        system "clear";
+        $ENV{PATH} = $save_path;
+    }
+}
+
+# ----------------------------------------------------------------------
+# Accessor functions
+# ----------------------------------------------------------------------
+
+sub compat(;$)        { shift()->accessor('-compat',          shift()) }
+sub clear_on_exit(;$) { shift()->accessor('-clear_on_exit',   shift()) }
+sub cursor_mode(;$)   { shift()->accessor('-cursor_mode',     shift()) }
+sub lang(;$)          { shift()->accessor('-language_object', shift()) }
+
+# TODO: document
+sub debug(;$)         
+{ 
+    my $this  = shift;
+    my $value = shift;
+    $Curses::UI::debug = $this->accessor('-debug', $value);
+}
+
+# ----------------------------------------------------------------------
+# Window resizing support
+# ----------------------------------------------------------------------
 
 sub layout()
 {
-	my $this = shift;
-	return $this if $Curses::UI::screen_too_small;
+    my $this = shift;
+
+    return $this if $Curses::UI::initialized;
+
+    $Curses::UI::screen_too_small = 0;
+
+    # Initialize the curses screen.
+    initscr();
+    noecho();
+    raw();
+
+    # Mouse events if possible
+    my $old = 0;
+    if ( $Curses::UI::ncurses_mouse ) 
+    {
+	print STDERR "DEBUG: ncurses mouse events are enabled\n"
+	    if $Curses::UI::debug;
+        # In case of gpm, mousemask fails. 
+	eval { mousemask( ALL_MOUSE_EVENTS(), $old ) };
+        print STDERR "mousemask() failed: $@\n" if $@;
+    }   
 	
-	if (defined $this->{-scr})
-	{
-		delwin($this->{-windowscr});
-		delete $this->{-windowscr};
-		delete $this->{-scr};
+    # find the terminal size.
+    my ($cols,$lines) = GetTerminalSize;
+    $ENV{COLS}  = $cols;
+    $ENV{LINES} = $lines;
 
-		endwin();
-	}
+    # Create root window.
+    my $root = newwin($lines, $cols, 0, 0);
+    die "newwin($lines, $cols, 0, 0) failed\n"
+	unless defined $root;
 
-	my ($cols,$lines) = GetTerminalSize;
-	$ENV{COLS} = $cols;
-	$ENV{LINES} = $lines;
+    # Let this object present itself as a standard 
+    # Curses::UI widget, regarding size, location and
+    # drawing area. This will make it possible for
+    # child windows / widgets to layout and draw themselves.
+    #    
+    $this->{-width}  = $this->{-w} = $this->{-bw} = $cols;
+    $this->{-height} = $this->{-h} = $this->{-bh} = $lines;
+    $this->{-x}      = $this->{-y} = 0;
+    $this->{-canvasscr} = $root;
 
-	initscr();
-	my $root = newwin($lines, $cols, 0, 0);
-
-	$this->{-width}  = $this->{-w} = $this->{-bw} = $cols;
-	$this->{-height} = $this->{-h} = $this->{-bh} = $lines;
-	$this->{-x} = $this->{-y} = 0;
-	$this->{-scr} = $root;
-	$this->{-windowscr} = $root;
-
-	$this->layout_contained_objects;
-	
-	return $this;	
+    # Walk through all contained objects and let them 
+    # layout themselves.
+    $this->layout_contained_objects;
+    
+    $Curses::UI::initialized = 1;
+    
+    return $this;    
 }
 
+# ----------------------------------------------------------------------
+# Event handling
+# ----------------------------------------------------------------------
 
-sub check_for_too_small_screen()
+# Tk-like mainloop, just for fun :-)
+sub MainLoop ()
 {
-	my $this = shift;
-	
-	if ($Curses::UI::screen_too_small) 
-	{
-		my $s = $this->rootscr;
-		$s->clear;
-		$s->addstr(0,0,"Your screen is currently too small "
-			     . "for this application.");
-		$s->addstr(1,0,"Resize the screen or press "
-                             . "<CTRL+C> to exit");
-		$s->move(2,0);
-		$s->noutrefresh();
-		doupdate();
-
-		for (;;) 
-		{
-			$this->check_for_resize;
-			last unless $Curses::UI::screen_too_small;
-			sleep 1;
-		}
-	
-	}
-
-	return $this;
+    die "No Curses::UI rootobject available!\n"
+        unless defined $Curses::UI::rootobject;
+    $Curses::UI::rootobject->mainloop;
 }
 
-sub rebuild()
+# TODO: document
+sub mainloop ()
 {
-	my $this = shift;
-	$this->check_for_too_small_screen();
-	$this->SUPER::rebuild();
+    my $this = shift;
+
+    # Draw the initial screen.
+    $this->focus(undef, 1); # 1 = forced focus
+    $this->draw;
+    doupdate();
+
+    # Inifinite event loop.
+    for(;;) 
+    { 
+        $this->do_one_event 
+    }
+}
+
+# TODO: document
+sub do_one_event(;$)
+{
+    my $this = shift;
+    my $object = shift;
+    $object = $this unless defined $object;
+        
+    eval {curs_set($this->{-cursor_mode})};
+
+    # Read a key or use the feeded key.
+    my $key = $this->{-feedkey};
+    unless (defined $key) {
+        $key = $this->get_key($this->{-read_timeout});
+    }
+    $this->{-feedkey} = undef;
+
+    # ncurses sends KEY_RESIZE() key on resize. Ignore this key.
+    if (Curses->can("KEY_RESIZE")) {
+        $key = '-1' if $key eq KEY_RESIZE();
+    }
+
+    # ncurses sends KEY_MOUSE()
+    if ($Curses::UI::ncurses_mouse) {
+	if ($key eq KEY_MOUSE()) {
+	    $this->handle_mouse_event($object);
+	    doupdate();
+	    return $this;
+	}
+    }
+
+    # If the screen is too small, then <CTRL+C> will exit.
+    # Else the next event loop will be started.
+    if ($Curses::UI::screen_too_small) {
+	exit(1) if $key eq "\cC";
 	return $this;
+    }
+
+    # Delegate the keypress. This is not done to $this,
+    # but to $object, so all events will go to the 
+    # object that called do_one_event(). This is used to
+    # enable modal focusing. 
+    #
+    $object->event_keypress($key) unless $key eq '-1';
+        
+    # Execute timer code.
+    $this->do_timer;
+
+    # See if there are pending keys on input. If I do not
+    # feed them to the application in this way, the screen
+    # hangs in case I do a lot of input on my Solaris
+    # machine.
+    $key = $this->get_key(0);
+    $this->feedkey($key) unless $key eq '-1';
+
+    # Update the screen.
+    doupdate();
+
+    return $this;
 }
 
 sub draw()
 {
-	my $this = shift;
-	$this->check_for_too_small_screen();
-	$this->SUPER::draw();
-	return $this;
+    my $this = shift;
+    my $no_doupdate = shift || 0;
+
+    if ($Curses::UI::screen_too_small) 
+    {
+        my $s = $this->{-canvasscr};
+        $s->clear;
+        $s->addstr(0, 0, $this->lang->get('screen_too_small'));
+        $s->move(4,0);
+        $s->noutrefresh();
+	doupdate();
+    } else {
+	$this->SUPER::draw(1);
+	doupdate() unless $no_doupdate;
+    }
 }
 
+# TODO: document
+sub feedkey()
+{
+    my $this = shift;
+    my $key = shift;
+    $this->flushkeys();
+    $this->{-feedkey} = $key;
+    return $this;
+}
+
+# TODO: document
+sub flushkeys()
+{
+    my $this = shift;
+    
+    my $key = '';
+    my @k = ();
+    until ( $key eq "-1" ) {
+        $key = $this->get_key(0);
+    }
+}
+
+
+# ----------------------------------------------------------------------
+# Timed event handling
+# ----------------------------------------------------------------------
+
+sub set_read_timeout()
+{
+    my $this = shift;
+
+    my $new_timeout = -1;
+    TIMER: while (my ($id, $config) = each %{$this->{-timers}}) 
+    {
+        # Skip timer if it is disabled.
+        next TIMER unless $config->{-enabled};
+
+	$new_timeout = $config->{-time} 
+	    unless $new_timeout != -1 and
+	           $new_timeout < $config->{-time};
+    }
+    $new_timeout = 1 if $new_timeout < 0 and $new_timeout != -1;
+
+    $this->{-read_timeout} = $new_timeout;
+    return $this;
+}
+
+sub set_timer($$;)
+{
+    my $this     = shift;
+    my $id       = shift;
+    my $callback = shift;
+    my $time     = shift || 1;
+
+    $this->fatalerror(
+        "add_timer(): callback is no CODE reference"
+    ) unless defined $callback and ref $callback eq 'CODE';
+
+    $this->fatalerror(
+	"add_timer(): id is not set"
+    ) unless defined $id;
+
+    my $config = {
+        -time     => int($time),
+        -callback => $callback,
+        -enabled  => 1,
+        -lastrun  => time(),
+    };
+    $this->{-timers}->{$id} = $config;
+
+    $this->set_read_timeout;
+
+    return $this;
+}
+
+sub disable_timer($;)
+{
+    my ($this,$id) = @_;
+    if (defined $this->{-timers}->{$id}) {
+        $this->{-timers}->{$id}->{-enabled} = 0;
+    }    
+    $this->set_read_timeout;
+    return $this;
+}
+
+sub enable_timer($;)
+{
+    my ($this,$id) = @_;
+    if (defined $this->{-timers}->{$id}) {
+        $this->{-timers}->{$id}->{-enabled} = 1;
+    }    
+    $this->set_read_timeout;
+    return $this;
+}
+
+sub delete_timer($;)
+{
+    my ($this,$id) = @_;
+    if (defined $this->{-timers}->{$id}) {
+        delete $this->{-timers}->{$id};
+    }    
+    $this->set_read_timeout;
+    return $this;
+}
+
+sub do_timer()
+{
+    my $this = shift;
+
+    my $now = time();
+    my $timers_done = 0;
+
+    TIMER: while (my ($id, $config) = each %{$this->{-timers}}) 
+    {
+        # Skip timer if it is disabled.
+        next TIMER unless $config->{-enabled};
+
+        # No -lastrun set? Then do it now.
+        unless (defined $config->{-lastrun})
+        {
+            $config->{-lastrun} = $now; 
+            next TIMER;
+        }
+
+        if ($config->{-lastrun} <= ($now - $config->{-time})) 
+        { 
+            $config->{-callback}->($this);
+            $config->{-lastrun} = $now;
+            $timers_done++;
+        }
+    }
+
+    # Bring the cursor back to the focused object by
+    # redrawing it. Due to drawing other objects, it might 
+    # have moved to another widget or screen location.
+    #
+    $this->focus_path(-1)->draw if $timers_done;
+        
+    return $this;
+}
+
+# ----------------------------------------------------------------------
+# Mouse events
+# ----------------------------------------------------------------------
+
+sub handle_mouse_event()
+{
+    my $this = shift;
+    my $object = shift;
+    $object = $this unless defined $object;
+
+    my $MEVENT = 0;
+    getmouse($MEVENT);
+
+    # $MEVENT is a struct. From curses.h (note: this might change!):
+    #
+    # typedef struct
+    # {
+    #    short id;           /* ID to distinguish multiple devices */
+    #	 int x, y, z;        /* event coordinates (character-cell) */
+    #	 mmask_t bstate;     /* button state bits */
+    # } MEVENT;
+    #
+    # ---------------
+    # s signed short
+    # x null byte
+    # x null byte
+    # ---------------
+    # i integer
+    # ---------------
+    # i integer
+    # ---------------
+    # i integer
+    # ---------------
+    # l long
+    # ---------------
+
+    my ($id, $x, $y, $z, $bstate) = unpack("sx2i3l", $MEVENT);
+    my %MEVENT = (
+	-id     => $id,
+	-x      => $x,
+	-y      => $y,
+	-bstate => $bstate
+    );
+
+    # Get the objects at the mouse event position.
+    my $tree = $this->object_at_xy($object, $MEVENT{-x}, $MEVENT{-y});
+   
+    # Walk through the object tree, top object first.
+    foreach my $object (reverse @$tree)
+    {
+	# Send the mouse-event to the object. 
+	# Leave the loop if the object handled the event.
+	my $return = $object->event_mouse(\%MEVENT);
+	last if defined $return and $return ne 'DELEGATE';
+    }
+}
+
+sub object_at_xy($$;$)
+{
+    my $this = shift;
+    my $object = shift;
+    my $x = shift;
+    my $y = shift;
+    my $tree = shift;
+    $tree = [] unless defined $tree;
+
+    push @$tree, $object;
+
+    my $idx = -1;
+    while (defined $object->{-draworder}->[$idx])
+    {
+        my $testobj = $object->getobj($object->{-draworder}->[$idx]);
+        $idx--;
+
+        # Find the window parameters for the $testobj.
+        my $scr = defined $testobj->{-borderscr} ? '-borderscr' : '-canvasscr';
+        my $winp = $testobj->windowparameters($scr);
+
+        # Does the click fall inside this object?
+        if ( $x >= $winp->{-x} and
+             $x <  ($winp->{-x}+$winp->{-w}) and
+             $y >= $winp->{-y} and
+             $y <  ($winp->{-y}+$winp->{-h}) ) {
+
+            if ( $testobj->isa('Curses::UI::Container') and
+                 not $testobj->isa('Curses::UI::ContainerWidget')) {
+                $this->object_at_xy($testobj, $x, $y, $tree)
+            } else {
+                push @$tree, $testobj;
+            }
+            return $tree;
+        }
+    }
+
+    return $tree;
+}
+
+
+# ----------------------------------------------------------------------
+# Other subroutines
+# ----------------------------------------------------------------------
+
+# TODO: document
+sub fatalerror($$;$)
+{
+    my $this  = shift;
+    my $error = shift;
+    my $exit  = shift;
+
+    $exit = 1 unless defined $exit;
+    chomp $error; 
+    $error .= "\n";
+
+    my $s = $this->{-canvasscr};
+    $s->clear;
+    $s->addstr(0,0,"Fatal program error:\n"
+    	     . "-"x($ENV{COLS}-1) . "\n"
+    	     . $error 
+    	     . "-"x($ENV{COLS}-1) . "\n"
+    	     . "Press any key to exit...");
+    $s->noutrefresh();
+    doupdate();
+
+    $this->flushkeys();
+    for (;;) 
+    {
+	$key = $this->get_key();
+	last if $key ne "-1";
+    } 
+
+    exit($exit);
+}
+
+# TODO: document
+sub usemodule($;)
+{
+    my $this = shift;
+    my $class = shift;
+
+    # Create class filename.
+    my $file = $class;
+    $file =~ s|::|/|g; 
+    $file .= '.pm';
+
+    # Automatically load the required class.
+    if (not defined $INC{$file})
+    {
+        eval 
+	{
+            require $file;
+            $class->import;
+        };
+
+        # Fatal error if the class could not be loaded.
+	$this->fatalerror("Could not load $class from $file:\n$@")
+	    if $@;
+    }
+
+    return $this;
+}
+
+sub focus_path()
+{
+    my $this = shift;
+    my $index = shift;
+
+    my $p_obj = $this;
+    my @path = ($p_obj);
+    for(;;)
+    {
+        my $p_el = $p_obj->{-draworder}->[-1];
+        last unless defined $p_el;
+        $p_obj = $p_obj->{-id2object}->{$p_el};
+        push @path, $p_obj;
+        last if $p_obj->isa('Curses::UI::ContainerWidget');
+    }
+
+    return (defined $index ? $path[$index] : @path);
+}
+
+# add() is overridden, because we only want to be able
+# to add Curses::UI:Window objects to the Curses::UI
+# rootlevel.
+#
 sub add()
 {
-	my $this = shift;
-	my $id = shift;
-	my $class = shift;
-	my %args = @_;
-	
-        # Make it possible to specify WidgetType instead of
-        # Curses::UI::WidgetType.
-        $class = "Curses::UI::$class" 
-		if $class !~ /\:\:/ 
-		or $class =~ /^Dialog\:\:[^\:]+$/;
+    my $this = shift;
+    my $id = shift;
+    my $class = shift;
+    my %args = @_;
+    
+    # Make it possible to specify WidgetType instead of
+    # Curses::UI::WidgetType.
+    $class = "Curses::UI::$class" 
+        if $class !~ /\:\:/ or 
+           $class =~ /^Dialog\:\:[^\:]+$/;
 
-	$this->SUPER::usemodule($class);
+    $this->usemodule($class);
 
-	confess "You may only add Curses::UI::Window objects to "
-	  . "Curses::UI and no $class objects"
-		unless $class->isa('Curses::UI::Window');
-	
-	$this->SUPER::add($id, $class, %args);
+    $this->fatalerror(
+	    "You may only add Curses::UI::Window objects to "
+          . "Curses::UI and no $class objects"
+    ) unless $class->isa('Curses::UI::Window');
 
+    $this->SUPER::add($id, $class, %args);
 }
 
-sub tempscreen()
-{
-	my $this = shift;
-	my $id = shift;
-	my $class = shift;
-	my %args = @_;
+# ----------------------------------------------------------------------
+# Focusable dialog windows
+# ----------------------------------------------------------------------
 
-	my $tmp = $this->add($id, $class, %args, -parent => $this);
-	$this->focus_to_object($id);
-	$this->focus_object;
-	$return = $tmp->get;
-	$this->delete($id);
-	$this->rebuild;
-	return $return;
+sub tempdialog()
+{
+    my $this = shift;
+    my $class = shift;
+    my %args = @_;
+
+    my $id = "__window_$class";
+
+    my $dialog = $this->add($id, $class, %args);
+    $dialog->modalfocus;
+    $return = $dialog->get;
+    $this->delete($id);
+    $this->root->focus(undef, 1);
+    return $return;
+}
+
+# The argument list will be returned unchanged, unless it
+# contains only one item. In that case ($ifone, $_[0]) will
+# be returned. This enables constructions like:
+#
+#    $cui->dialog("Some dialog message");
+#
+# instead of:
+#
+#    $cui->dialog(-message => "Some dialog message");
+#
+sub process_args($$;)
+{
+    my $this = shift;        
+    my $ifone = shift;
+    if (@_ == 1) { @_ = ($ifone => $_[0]) }
+    return @_;
 }
 
 sub error()
 {
-	my $this = shift;
-
-	# make ->error("message") possible.
-	if (@_ == 1) { @_ = (-message => $_[0]) } 
-	my %args = @_;
-
-	$this->tempscreen(
-		'_error',
-		'Dialog::Error',
-		%args
-        );
+    my $this = shift;
+    my %args = $this->process_args('-message', @_);
+    $this->tempdialog('Dialog::Error', %args);
 }
 
 sub dialog()
 {
-	my $this = shift;
-	
-	# make ->error("message") possible.
-	if (@_ == 1) { @_ = (-message => $_[0]) } 
-	my %args = @_;
+    my $this = shift;
+    my %args = $this->process_args('-message', @_);
+    $this->tempdialog('Dialog::Basic', %args);
+}
 
-	$this->tempscreen(
-		'_dialog',
-		'Dialog::Basic',
-		%args
-        );
+sub calendardialog()
+{
+    my $this = shift;
+    my %args = $this->process_args('-title', @_);
+    $this->tempdialog('Dialog::Calendar', %args);
 }
 
 sub filebrowser()
 {
-	my $this = shift;
-	my %args = @_;
-	$this->tempscreen(
-		'_filebrowser',
-		'Dialog::Filebrowser',
-		%args
-        );
+    my $this = shift;
+    my %args = $this->process_args('-title', @_);
+
+    # Create title
+    unless (defined $args{-title}) {
+	my $l = $this->root->lang;
+	$args{-title} = $l->get('file_title');
+    }
+
+    # Select a file to load from.
+    $this->tempdialog('Dialog::Filebrowser', %args);
 }
 
 sub savefilebrowser()
 {
-	my $this = shift;
-	my %args = @_;
+    my $this = shift;
+    my %args = $this->process_args('-title', @_);
+    
+    my $l = $this->root->lang;
 
-	# Select a file to save to.
-	my $file = $this->filebrowser(
-		-editfilename 	=> 1,
-		%args,
-	);
+    # Create title.
+    $args{-title} = $l->get('file_savetitle')
+	unless defined $args{-title};
+    
+    # Select a file to save to.
+    my $file = $this->filebrowser(-editfilename => 1, %args);
+    return unless defined $file;
+    
+    # Check if the file exists. Ask for overwrite
+    # permission if it does.
+    if (-e $file)
+    {
+	# Get language specific data.
+	my $pre = $l->get('file_overwrite_question_pre');
+	my $post = $l->get('file_overwrite_question_post');
+	my $title = $l->get('file_overwrite_title');
 
-	return unless defined $file;
-	
-	# Check if the file exists. Ask for overwrite
-	# permission if it does.
-	if (-e $file)
-	{
-		my $overwrite = $this->dialog(
-			-title     => "Confirm overwrite",
-			-buttons   => ['< Yes >', '< No >'],
-			-values    => [1        , 0       ],
-			-shortcuts => ['y'      , 'n'     ],
-			-message   => "Do you really want to overwrite\n"
-                                    . "the file \"$file\"?"
-		);
-		return unless $overwrite;
-	}
-	
-	return $file;
+        my $overwrite = $this->dialog(
+            -title     => $title,
+            -buttons   => [ 'yes', 'no' ],
+            -message   => $pre . $file . $post,
+        );
+        return unless $overwrite;
+    }
+    
+    return $file;
 }
 
 sub loadfilebrowser()
 {
-	my $this = shift;
-	my %args = @_;
+    my $this = shift;
+    my %args = $this->process_args('-title', @_);
 
-	# Select a file to load from.
-	my $file = $this->filebrowser(
-		-editfilename 	=> 0,
-		%args,
-	);
+    # Create title
+    unless (defined $args{-title}) {
+	my $l = $this->root->lang;
+	$args{-title} = $l->get('file_loadtitle');
+    }
+
+    $this->filebrowser(-editfilename  => 0, %args);
 }
 
+# ----------------------------------------------------------------------
+# Non-focusable dialogs
+# ----------------------------------------------------------------------
+
+my $status_id = "__status_dialog";
 sub status($;)
 {
-	my $this = shift;
+    my $this = shift;
+    my %args = $this->process_args('-message', @_);
 
-	# make ->error("message") possible.
-	if (@_ == 1) { @_ = (-message => $_[0]) } 
-	my %args = @_;
-
-	$this->delete('_status');
-	$this->add(
-		'_status', 'Dialog::Status',
-		%args,
-	);
-
-	$this->draw;
-	return $this;	
+    $this->delete($status_id);
+    $this->add($status_id, 'Dialog::Status', %args)->draw;
+    
+    return $this;    
 }
 
 sub nostatus()
 {
-	my $this = shift;
-	$this->delete('_status');
-	$this->draw;
-	return $this;
+    my $this = shift;
+    $this->delete($status_id);
+    $this->flushkeys();
+    $this->draw;
+    return $this;
 }
 
 sub progress()
 {
-	my $this = shift;
-	my %args = @_;
+    my $this = shift;
+    my %args = @_;
 
-	$this->add(
-		'_progress', 'Dialog::Progress',
-		%args,
-	);
-	$this->draw;
+    $this->add(
+        "__progress_$this",
+        'Dialog::Progress', 
+        %args
+    );
+    $this->draw;
 
-	return $this;
+    return $this;
 }
 
 sub setprogress($;$)
 {
-	my $this = shift;
-	my $pos  = shift;
-	my $message = shift;
+    my $this = shift;
+    my $pos  = shift;
+    my $message = shift;
 
-	my $p = $this->getobj('_progress');
-	return unless defined $p;
-	$p->pos($pos) if defined $pos;
-	$p->message($message) if defined $message;
-	$p->draw;	
+    # If I do not do this, the progress bar seems frozen
+    # if a key is pressed on my Solaris machine. Flushing
+    # the input keys solves this. And this is not a bad
+    # thing to do during a progress dialog (input is ignored
+    # this way).
+    $this->flushkeys;
 
-	return $this;
+    my $p = $this->getobj("__progress_$this");
+    return unless defined $p;
+    $p->pos($pos) if defined $pos;
+    $p->message($message) if defined $message;
+    $p->draw;
+
+    return $this;
 }
 
 sub noprogress()
 {
-	my $this = shift;
-	$this->delete('_progress');
-	$this->draw;
-	return $this;
-}
-
-DESTROY 
-{ 
-	my $this = shift;
-	endwin();
-		
-	if ($this->{-clear_on_exit})
-	{
-		my $save_path = $ENV{PATH};
-		$ENV{PATH} = "/bin:/usr/bin";
-		system "clear";
-		$ENV{PATH} = $save_path;
-	}
+    my $this = shift;
+    $this->delete("__progress_$this");
+    $this->flushkeys;
+    $this->draw;
+    return $this;
 }
 
 1;
@@ -571,10 +1054,10 @@ Example:
         unless (open F, ">$file") {
             print F "Hello, world!\n";
             close F;
-	} else {
+    } else {
             $cui->error("Error on writing to "
                        ."\"$file\":\n$!");
-	}
+    }
     } 
 
 =item B<loadfilebrowser>( OPTIONS )
@@ -641,12 +1124,12 @@ Example:
 
     $cui->progress(
         -max => 10,
-	-message => "Counting 10 seconds...",
+    -message => "Counting 10 seconds...",
     );
 
     for my $second (0..10) {
-	$cui->setprogress($second)
-	sleep 1;
+    $cui->setprogress($second)
+    sleep 1;
     }
 
     $cui->noprogress;
@@ -778,9 +1261,9 @@ if this is not the desired behaviour.
 
 If you want the buttons themselves to loose focus if 
 pressed, then change its routine for the "return" 
-binding from "LEAVE_CONTAINER" to "RETURN". Example:
+binding from "LEAVE_CONTAINER" to "LOOSE_FOCUS". Example:
 
-    $but1->set_routine('return', 'RETURN');
+    $but1->set_routine('loose-focus', 'LOOSE_FOCUS');
 
 To make things a bit more snappy, we want to add some
 shortcut keys to the appliction:
@@ -825,7 +1308,7 @@ on them, we will have to make things work like they should.
             # shortcut keys we created using returnkeys().
             if ($lastkey eq "\cN") {
                 next WINDOW;
-	    } elsif ($lastkey eq "\cQ") {
+        } elsif ($lastkey eq "\cQ") {
                 last MAINLOOP;
             }
 
