@@ -9,6 +9,8 @@
 # e-mail: maurice@gitaar.net
 # ----------------------------------------------------------------------
 
+# TODO: fix dox
+
 package Curses::UI::TextEntry;
 
 use strict;
@@ -16,43 +18,51 @@ use Curses;
 use Curses::UI::Common;
 use Curses::UI::TextEditor;
 
-use vars qw($VERSION @ISA);
-@ISA = qw(Curses::UI::TextEditor);
-$VERSION = '1.00';
-	
+use vars qw(
+    $VERSION 
+    @ISA
+);
+
+$VERSION = '1.10';
+
+@ISA = qw(
+    Curses::UI::TextEditor
+);
+    
 sub new ()
 {
-	my $class = shift;
+    my $class = shift;
 
-        my %userargs = @_;
-        keys_to_lowercase(\%userargs);
+    my %userargs = @_;
+    keys_to_lowercase(\%userargs);
 
-	my %args = ( 
-		-undolevels	 => 20,	# number of undolevels. 0 = infinite
-		-homeonreturn    => 1,	# cursor to homepos on return?
+    my %args = ( 
+        -undolevels      => 20, # number of undolevels. 0 = infinite
+        -homeonblur      => 1,  # cursor to homepos on blur?
 
-		%userargs,
+        %userargs,
 
-		-singleline	 => 1,	# single line mode or not?
-		-showhardreturns => 0,	# show hard returns with diamond char?
-	);
+        -singleline      => 1,  # single line mode or not?
+        -showhardreturns => 0,  # show hard returns with diamond char?
+    );
 
-	# Create the entry.
-	my $this = $class->SUPER::new( %args);
+    # Create the entry.
+    my $this = $class->SUPER::new( %args );
 
-	# There is no reason to show overflow symbols if no
-	# more characters than the available width can be
-	# added (the screen would wrap and after that
-	# typing would be impossible).
-	if ($this->{-maxlength} and $this->screenwidth > $this->{-maxlength}) {
-		$this->{-showoverflow} = 0;
-	}
+    # There is no reason to show overflow symbols if no
+    # more characters than the available width can be
+    # added (the screen would wrap and after that
+    # typing would be impossible).
+    if ($this->{-maxlength} and 
+	$this->canvaswidth > $this->{-maxlength}) {
+        $this->{-showoverflow} = 0;
+    }
 
-	# Setup bindings.
-        $this->clear_binding('return');
-        $this->set_binding('return', KEY_ENTER(), "\t" );
+    # Setup bindings.
+    $this->clear_binding('loose-focus');
+    $this->set_binding('loose-focus', KEY_ENTER(), CUI_TAB(), KEY_BTAB() );
 
-	return bless $this, $class;
+    return $this;
 }
 
 1;
